@@ -1,6 +1,6 @@
 package Git::Raw::Commit;
 {
-  $Git::Raw::Commit::VERSION = '0.16';
+  $Git::Raw::Commit::VERSION = '0.17';
 }
 
 use strict;
@@ -14,7 +14,7 @@ Git::Raw::Commit - Git commit class
 
 =head1 VERSION
 
-version 0.16
+version 0.17
 
 =head1 DESCRIPTION
 
@@ -24,13 +24,13 @@ A C<Git::Raw::Commit> represents a Git commit.
 
 =head2 create( $repo, $msg, $author, $committer, [@parents], $tree )
 
-Create a new commit given a message, a L<Git::Raw::Signature> representing the
-commit author and one representing the committer, a list of parent commits and
-a L<Git::Raw::Tree>.
+Create a new commit given a message, two L<Git::Raw::Signature> (one is the
+commit author and the other the committer), a list of parent commits and a
+L<Git::Raw::Tree>.
 
 =head2 lookup( $repo, $id )
 
-Retrieve the commit corresponding to the given id. This function is pretty much
+Retrieve the commit corresponding to C<$id>. This function is pretty much
 the same as C<$repo-E<gt>lookup($id)> except that it only returns commits.
 
 =head2 id( )
@@ -39,32 +39,31 @@ Retrieve the id of the commit, as string.
 
 =head2 message( )
 
-Retrieve the commit's message.
+Retrieve the message of the commit.
 
 =head2 author( )
 
-Retrieve the C<Git::Raw::Signature> representing the commit's author.
+Retrieve the L<Git::Raw::Signature> representing the author of the commit.
 
 =head2 committer( )
 
-Retrieve the C<Git::Raw::Signature> representing the commit's committer.
+Retrieve the L<Git::Raw::Signature> representing the committer.
 
 =head2 time( )
 
-Retrieve the committer time of a commit.
+Retrieve the committer time of the commit.
 
 =head2 offset( )
 
-Retrieve the committer time offset (in minutes) of a commit.
+Retrieve the committer time offset (in minutes) of the commit.
 
 =head2 tree( )
 
-Retrieve the L<Git::Raw::Tree> a commit points to.
+Retrieve the L<Git::Raw::Tree> the commit points to.
 
 =head2 parents( )
 
-Retrieve the list of parents of the commit. The parents are C<Git::Raw::Commit>
-too,
+Retrieve the list of parents of the commit.
 
 =head1 AUTHOR
 
