@@ -1,6 +1,6 @@
 package Git::Raw::Config;
 {
-  $Git::Raw::Config::VERSION = '0.16';
+  $Git::Raw::Config::VERSION = '0.17';
 }
 
 use strict;
@@ -14,7 +14,7 @@ Git::Raw::Config - Git config class
 
 =head1 VERSION
 
-version 0.16
+version 0.17
 
 =head1 DESCRIPTION
 
@@ -22,24 +22,46 @@ A C<Git::Raw::Config> represents a Git configuration file.
 
 =head1 METHODS
 
+=head2 new( )
+
+Create a new config object.
+
+=head2 add_file( $path, $level )
+
+Add C<$path> to the config object with priority level C<$level>.
+
 =head2 bool( $name [, $value ] )
 
 Retrieve the value of the C<$name> configuration field of type boolean. If
-C<$value> is passed, the value of the configration will be updated, and
+C<$value> is passed, the value of the configration will be updated and
 returned.
 
 =head2 int( $name [, $value ] )
 
 Retrieve the value of the C<$name> configuration field of type integer. If
-C<$value> is passed, the value of the configration will be updated, and
+C<$value> is passed, the value of the configration will be updated and
 returned.
 
 =head2 str( $name [, $value ] )
 
 Retrieve the value of the C<$name> configuration field of type string. If
-C<$value> is passed, the value of the configration will be updated, and
+C<$value> is passed, the value of the configration will be updated and
 returned.
 
+=head2 foreach( $callback )
+
+Run C<$callback> for every config entry. The callback receives the name of the
+config entry, its value and its priority level. A non-zero return value stops
+the loop.
+
+=head2 refresh( )
+
+Reload the config files from disk.
+
+=head2 delete( $name )
+
+Delete the variable C<$name> from the config object.
+
 =head1 AUTHOR
 
 Alessandro Ghedini <alexbio@cpan.org>
