@@ -1,6 +1,6 @@
 package Git::Raw::Repository;
 {
-  $Git::Raw::Repository::VERSION = '0.16';
+  $Git::Raw::Repository::VERSION = '0.17';
 }
 
 use strict;
@@ -14,7 +14,7 @@ Git::Raw::Repository - Git repository class
 
 =head1 VERSION
 
-version 0.16
+version 0.17
 
 =head1 DESCRIPTION
 
@@ -26,9 +26,10 @@ A C<Git::Raw::Repository> represents a Git repository.
 
 Initialize a new repository at C<$path>.
 
-=head2 clone( $url, $path, $is_bare )
+=head2 clone( $url, $path, \%strategy, $is_bare )
 
-Clone the repository at C<$url> to C<$path>.
+Clone the repository at C<$url> to C<$path>. See the C<checkout()> description
+for more information about C<%strategy>.
 
 =head2 open( $path )
 
@@ -52,32 +53,64 @@ Retrieve the object pointed by the HEAD of the repository.
 
 =head2 lookup( $id )
 
-Retrieve the object corresponding to the given id.
+Retrieve the object corresponding to C<$id>.
+
+=head2 checkout( $object, \%strategy )
+
+Updates the files in the index and working tree to match the content of
+C<$object>. The C<%strategy> paramenter can have set the following values:
+
+=over 4
+
+=item * "update_unmodified"
+
+=item * "update_missing"
+
+=item * "update_modified"
+
+=item * "update_untracked"
+
+=item * "allow_conflicts"
+
+=item * "skip_unmerged"
+
+=item * "update_only"
+
+=item * "remove_untracked"
+
+=back
+
+Example:
+
+    $repo -> checkout($repo -> head, {
+      'update_missing'  => 1,
+      'update_modified' => 1
+    });
 
 =head2 reset( $target, $type )
 
 Reset the current HEAD to the given commit. Valid reset types are: C<"soft">
-(the head will be moved to the commit), C<"mixed"> (trigger a soft reset and
+(the head will be moved to the commit) or C<"mixed"> (trigger a soft reset and
 replace the index with the content of the commit tree).
 
 =head2 status( $file )
 
-Retrieve the status of the given file in the working directory. This functions
-returns a list of status flags. Valid status flags are: C<"index_new">,
+Retrieve the status of <$file> in the working directory. This functions returns
+a list of status flags. Possible status flags are: C<"index_new">,
 C<"index_modified">, C<"index_deleted">, C<"worktree_new">,
 C<"worktree_modified">, C<"worktree_deleted"> and C<"ignored">.
 
 =head2 ignore( $rules )
 
-Add ignore rules to the repository. The format of the rules is the same one of
-the C<.gitignore> file (see the C<gitignore(5)> manpage). Example:
+Add an ignore rules to the repository. The format of the rules is the same one
+of the C<.gitignore> file (see the C<gitignore(5)> manpage). Example:
 
     $repo -> ignore("*.o\n");
 
 =head2 diff( $repo [, $tree] )
 
-Compute the L<Git::Raw::Diff> between the repository index and a tree. If no
-C<$tree> is passed, the diff will be computed against the working directory.
+Compute the L<Git::Raw::Diff> between the repository default index and a tree.
+If no C<$tree> is passed, the diff will be computed against the working directory.
 
 =head2 blob( $buffer )
 
@@ -112,7 +145,7 @@ sub branches {
 	return $branches;
 }
 
-=head2 commit( $msg, $author, $committer, [@parents], $tree )
+=head2 commit( $msg, $author, $committer, \@parents, $tree )
 
 Create a new L<Git::Raw::Commit>. Shortcut for C<Git::Raw::Commit-E<gt>create()>.
 
@@ -132,6 +165,19 @@ sub tag { return Git::Raw::Tag -> create(@_) }
 
 Retrieve a list of L<Git::Raw::Tag> objects.
 
+=cut
+
+sub tags {
+	my $self = shift;
+	my $tags;
+
+	Git::Raw::Tag -> foreach($self, sub {
+		push @$tags, shift; 0
+	});
+
+	return $tags;
+}
+
 =head2 stash( $stasher, $msg )
 
 Save the local modifications to a new stash. Shortcut for C<Git::Raw::Stash-E<gt>save()>.
@@ -162,11 +208,11 @@ Retrieve the working directory of the repository.
 
 =head2 is_empty( )
 
-Tell whether the repository is empty or not.
+Check if the repository is empty.
 
 =head2 is_bare( )
 
-Tell whether the repository is bare or not.
+Check if the repository is bare.
 
 =head1 AUTHOR
 
