@@ -29,6 +29,7 @@ enum {
 	GIT_EBAREREPO = -8,
 	GIT_EORPHANEDHEAD = -9,
 	GIT_EUNMERGED = -10,
+	GIT_ENONFASTFORWARD = -11,
 
 	GIT_PASSTHROUGH = -30,
 	GIT_ITEROVER = -31,
@@ -60,6 +61,7 @@ typedef enum {
 	GITERR_SUBMODULE,
 	GITERR_THREAD,
 	GITERR_STASH,
+	GITERR_CHECKOUT,
 } git_error_t;
 
 /**
