@@ -12,6 +12,7 @@
 #include "git2/index.h"
 #include "git2/submodule.h"
 #include "buffer.h"
+#include "buf_text.h"
 #include "vector.h"
 #include "posix.h"
 #include "config_file.h"
@@ -66,13 +67,13 @@ __KHASH_IMPL(
 	str_hash_no_trailing_slash, str_equal_no_trailing_slash);
 
 static int load_submodule_config(git_repository *repo, bool force);
-static git_config_file *open_gitmodules(git_repository *, bool, const git_oid *);
+static git_config_backend *open_gitmodules(git_repository *, bool, const git_oid *);
 static int lookup_head_remote(git_buf *url, git_repository *repo);
 static int submodule_get(git_submodule **, git_repository *, const char *, const char *);
 static void submodule_release(git_submodule *sm, int decr);
 static int submodule_load_from_index(git_repository *, const git_index_entry *);
 static int submodule_load_from_head(git_repository*, const char*, const git_oid*);
-static int submodule_load_from_config(const char *, const char *, void *);
+static int submodule_load_from_config(const git_config_entry *, void *);
 static int submodule_load_from_wd_lite(git_submodule *, const char *, void *);
 static int submodule_update_config(git_submodule *, const char *, const char *, bool, bool);
 static void submodule_mode_mismatch(git_repository *, const char *, unsigned int);
@@ -201,7 +202,7 @@ int git_submodule_add_setup(
 	int use_gitlink)
 {
 	int error = 0;
-	git_config_file *mods = NULL;
+	git_config_backend *mods = NULL;
 	git_submodule *sm;
 	git_buf name = GIT_BUF_INIT, real_url = GIT_BUF_INIT;
 	git_repository_init_options initopt;
@@ -332,7 +333,7 @@ int git_submodule_add_finalize(git_submodule *sm)
 	assert(sm);
 
 	if ((error = git_repository_index__weakptr(&index, sm->owner)) < 0 ||
-		(error = git_index_add(index, GIT_MODULES_FILE, 0)) < 0)
+		(error = git_index_add_from_workdir(index, GIT_MODULES_FILE)) < 0)
 		return error;
 
 	return git_submodule_add_to_index(sm, true);
@@ -341,7 +342,7 @@ int git_submodule_add_finalize(git_submodule *sm)
 int git_submodule_add_to_index(git_submodule *sm, int write_index)
 {
 	int error;
-	git_repository *repo, *sm_repo;
+	git_repository *repo, *sm_repo = NULL;
 	git_index *index;
 	git_buf path = GIT_BUF_INIT;
 	git_commit *head;
@@ -371,7 +372,7 @@ int git_submodule_add_to_index(git_submodule *sm, int write_index)
 
 	memset(&entry, 0, sizeof(entry));
 	entry.path = sm->path;
-	git_index__init_entry_from_stat(&st, &entry);
+	git_index_entry__init_from_stat(&entry, &st);
 
 	/* calling git_submodule_open will have set sm->wd_oid if possible */
 	if ((sm->flags & GIT_SUBMODULE_STATUS__WD_OID_VALID) == 0) {
@@ -393,7 +394,7 @@ int git_submodule_add_to_index(git_submodule *sm, int write_index)
 	git_commit_free(head);
 
 	/* add it */
-	error = git_index_add2(index, &entry);
+	error = git_index_add(index, &entry);
 
 	/* write it, if requested */
 	if (!error && write_index) {
@@ -412,7 +413,7 @@ cleanup:
 int git_submodule_save(git_submodule *submodule)
 {
 	int error = 0;
-	git_config_file *mods;
+	git_config_backend *mods;
 	git_buf key = GIT_BUF_INIT;
 
 	assert(submodule);
@@ -513,7 +514,7 @@ int git_submodule_set_url(git_submodule *submodule, const char *url)
 	return 0;
 }
 
-const git_oid *git_submodule_index_oid(git_submodule *submodule)
+const git_oid *git_submodule_index_id(git_submodule *submodule)
 {
 	assert(submodule);
 
@@ -523,7 +524,7 @@ const git_oid *git_submodule_index_oid(git_submodule *submodule)
 		return NULL;
 }
 
-const git_oid *git_submodule_head_oid(git_submodule *submodule)
+const git_oid *git_submodule_head_id(git_submodule *submodule)
 {
 	assert(submodule);
 
@@ -533,7 +534,7 @@ const git_oid *git_submodule_head_oid(git_submodule *submodule)
 		return NULL;
 }
 
-const git_oid *git_submodule_wd_oid(git_submodule *submodule)
+const git_oid *git_submodule_wd_id(git_submodule *submodule)
 {
 	assert(submodule);
 
@@ -695,7 +696,7 @@ int git_submodule_open(
 
 	/* if we have opened the submodule successfully, let's grab the HEAD OID */
 	if (!error && !(submodule->flags & GIT_SUBMODULE_STATUS__WD_OID_VALID)) {
-		if (!git_reference_name_to_oid(
+		if (!git_reference_name_to_id(
 				&submodule->wd_oid, *subrepo, GIT_HEAD_FILE))
 			submodule->flags |= GIT_SUBMODULE_STATUS__WD_OID_VALID;
 		else
@@ -717,7 +718,7 @@ int git_submodule_reload(git_submodule *submodule)
 	git_index *index;
 	int pos, error;
 	git_tree *head;
-	git_config_file *mods;
+	git_config_backend *mods;
 
 	assert(submodule);
 
@@ -733,7 +734,7 @@ int git_submodule_reload(git_submodule *submodule)
 
 	pos = git_index_find(index, submodule->path);
 	if (pos >= 0) {
-		git_index_entry *entry = git_index_get(index, pos);
+		const git_index_entry *entry = git_index_get_byindex(index, pos);
 
 		if (S_ISGITLINK(entry->mode)) {
 			if ((error = submodule_load_from_index(repo, entry)) < 0)
@@ -782,7 +783,7 @@ int git_submodule_reload(git_submodule *submodule)
 		git_buf path = GIT_BUF_INIT;
 
 		git_buf_sets(&path, "submodule\\.");
-		git_buf_puts_escape_regex(&path, submodule->name);
+		git_buf_text_puts_escape_regex(&path, submodule->name);
 		git_buf_puts(&path, ".*");
 
 		if (git_buf_oom(&path))
@@ -974,11 +975,12 @@ static int submodule_config_error(const char *property, const char *value)
 }
 
 static int submodule_load_from_config(
-	const char *key, const char *value, void *data)
+	const git_config_entry *entry, void *data)
 {
 	git_repository *repo = data;
 	git_strmap *smcfg = repo->submodules;
 	const char *namestart, *property, *alternate = NULL;
+	const char *key = entry->name, *value = entry->value;
 	git_buf name = GIT_BUF_INIT;
 	git_submodule *sm;
 	bool is_path;
@@ -1055,7 +1057,7 @@ static int submodule_load_from_config(
 	else if (strcasecmp(property, "update") == 0) {
 		int val;
 		if (git_config_lookup_map_value(
-			_sm_update_map, ARRAY_SIZE(_sm_update_map), value, &val) < 0)
+			&val, _sm_update_map, ARRAY_SIZE(_sm_update_map), value) < 0)
 			return submodule_config_error("update", value);
 		sm->update_default = sm->update = (git_submodule_update_t)val;
 	}
@@ -1066,7 +1068,7 @@ static int submodule_load_from_config(
 	else if (strcasecmp(property, "ignore") == 0) {
 		int val;
 		if (git_config_lookup_map_value(
-			_sm_ignore_map, ARRAY_SIZE(_sm_ignore_map), value, &val) < 0)
+			&val, _sm_ignore_map, ARRAY_SIZE(_sm_ignore_map), value) < 0)
 			return submodule_config_error("ignore", value);
 		sm->ignore_default = sm->ignore = (git_submodule_ignore_t)val;
 	}
@@ -1117,7 +1119,7 @@ static int load_submodule_config_from_index(
 	git_iterator *i;
 	const git_index_entry *entry;
 
-	if ((error = git_iterator_for_index(&i, repo)) < 0)
+	if ((error = git_iterator_for_repo_index(&i, repo)) < 0)
 		return error;
 
 	error = git_iterator_current(i, &entry);
@@ -1186,14 +1188,14 @@ static int load_submodule_config_from_head(
 	return error;
 }
 
-static git_config_file *open_gitmodules(
+static git_config_backend *open_gitmodules(
 	git_repository *repo,
 	bool okay_to_create,
 	const git_oid *gitmodules_oid)
 {
 	const char *workdir = git_repository_workdir(repo);
 	git_buf path = GIT_BUF_INIT;
-	git_config_file *mods = NULL;
+	git_config_backend *mods = NULL;
 
 	if (workdir != NULL) {
 		if (git_buf_joinpath(&path, workdir, GIT_MODULES_FILE) != 0)
@@ -1204,7 +1206,7 @@ static git_config_file *open_gitmodules(
 			if (git_config_file__ondisk(&mods, path.ptr) < 0)
 				mods = NULL;
 			/* open should only fail here if the file is malformed */
-			else if (git_config_file_open(mods) < 0) {
+			else if (git_config_file_open(mods, GIT_CONFIG_LEVEL_LOCAL) < 0) {
 				git_config_file_free(mods);
 				mods = NULL;
 			}
@@ -1229,7 +1231,7 @@ static int load_submodule_config(git_repository *repo, bool force)
 	int error;
 	git_oid gitmodules_oid;
 	git_buf path = GIT_BUF_INIT;
-	git_config_file *mods = NULL;
+	git_config_backend *mods = NULL;
 
 	if (repo->submodules && !force)
 		return 0;
@@ -1315,7 +1317,7 @@ static int lookup_head_remote(git_buf *url, git_repository *repo)
 	/* remote should refer to something like refs/remotes/ORIGIN/BRANCH */
 
 	if (git_reference_type(remote) != GIT_REF_SYMBOLIC ||
-		git__prefixcmp(git_reference_target(remote), GIT_REFS_REMOTES_DIR) != 0)
+		git__prefixcmp(git_reference_symbolic_target(remote), GIT_REFS_REMOTES_DIR) != 0)
 	{
 		giterr_set(GITERR_SUBMODULE,
 			"Cannot resolve relative URL when HEAD is not symbolic");
@@ -1323,7 +1325,7 @@ static int lookup_head_remote(git_buf *url, git_repository *repo)
 		goto cleanup;
 	}
 
-	scan = tgt = git_reference_target(remote) + strlen(GIT_REFS_REMOTES_DIR);
+	scan = tgt = git_reference_symbolic_target(remote) + strlen(GIT_REFS_REMOTES_DIR);
 	while (*scan && (*scan != '/' || (scan > tgt && scan[-1] != '\\')))
 		scan++; /* find non-escaped slash to end ORIGIN name */
 
@@ -1377,7 +1379,7 @@ static int submodule_update_config(
 		goto cleanup;
 
 	if (!value)
-		error = git_config_delete(config, key.ptr);
+		error = git_config_delete_entry(config, key.ptr);
 	else
 		error = git_config_set_string(config, key.ptr, value);
 
@@ -1388,8 +1390,8 @@ cleanup:
 
 static int submodule_index_status(unsigned int *status, git_submodule *sm)
 {
-	const git_oid *head_oid  = git_submodule_head_oid(sm);
-	const git_oid *index_oid = git_submodule_index_oid(sm);
+	const git_oid *head_oid  = git_submodule_head_id(sm);
+	const git_oid *index_oid = git_submodule_index_id(sm);
 
 	if (!head_oid) {
 		if (index_oid)
@@ -1409,7 +1411,7 @@ static int submodule_wd_status(unsigned int *status, git_submodule *sm)
 	const git_oid *wd_oid, *index_oid;
 	git_repository *sm_repo = NULL;
 
-	/* open repo now if we need it (so wd_oid() call won't reopen) */
+	/* open repo now if we need it (so wd_id() call won't reopen) */
 	if ((sm->ignore == GIT_SUBMODULE_IGNORE_NONE ||
 		 sm->ignore == GIT_SUBMODULE_IGNORE_UNTRACKED) &&
 		(sm->flags & GIT_SUBMODULE_STATUS_IN_WD) != 0)
@@ -1418,8 +1420,8 @@ static int submodule_wd_status(unsigned int *status, git_submodule *sm)
 			return error;
 	}
 
-	index_oid = git_submodule_index_oid(sm);
-	wd_oid    = git_submodule_wd_oid(sm);
+	index_oid = git_submodule_index_id(sm);
+	wd_oid    = git_submodule_wd_id(sm);
 
 	if (!index_oid) {
 		if (wd_oid)
@@ -1454,7 +1456,7 @@ static int submodule_wd_status(unsigned int *status, git_submodule *sm)
 		if (sm->ignore == GIT_SUBMODULE_IGNORE_NONE)
 			opt.flags |= GIT_DIFF_INCLUDE_UNTRACKED;
 
-		error = git_diff_index_to_tree(sm_repo, &opt, sm_head, &diff);
+		error = git_diff_index_to_tree(&diff, sm_repo, sm_head, NULL, &opt);
 
 		if (!error) {
 			if (git_diff_num_deltas(diff) > 0)
@@ -1471,7 +1473,7 @@ static int submodule_wd_status(unsigned int *status, git_submodule *sm)
 
 		/* perform index-to-workdir diff on submodule */
 
-		error = git_diff_workdir_to_index(sm_repo, &opt, &diff);
+		error = git_diff_workdir_to_index(&diff, sm_repo, NULL, &opt);
 
 		if (!error) {
 			size_t untracked =
