@@ -36,7 +36,7 @@ void test_refs_create__symbolic(void)
 	git_buf_free(&ref_path);
 
 	/* Create and write the new symbolic reference */
-	cl_git_pass(git_reference_create_symbolic(&new_reference, g_repo, new_head_tracker, current_head_target, 0));
+	cl_git_pass(git_reference_symbolic_create(&new_reference, g_repo, new_head_tracker, current_head_target, 0));
 
 	/* Ensure the reference can be looked-up... */
 	cl_git_pass(git_reference_lookup(&looked_up_ref, g_repo, new_head_tracker));
@@ -49,7 +49,7 @@ void test_refs_create__symbolic(void)
 	cl_assert(git_reference_type(resolved_ref) == GIT_REF_OID);
 
 	/* ...and that it points to the current master tip */
-	cl_assert(git_oid_cmp(&id, git_reference_oid(resolved_ref)) == 0);
+	cl_assert(git_oid_cmp(&id, git_reference_target(resolved_ref)) == 0);
 	git_reference_free(looked_up_ref);
 	git_reference_free(resolved_ref);
 
@@ -58,7 +58,7 @@ void test_refs_create__symbolic(void)
 
 	cl_git_pass(git_reference_lookup(&looked_up_ref, repo2, new_head_tracker));
 	cl_git_pass(git_reference_resolve(&resolved_ref, looked_up_ref));
-	cl_assert(git_oid_cmp(&id, git_reference_oid(resolved_ref)) == 0);
+	cl_assert(git_oid_cmp(&id, git_reference_target(resolved_ref)) == 0);
 
 	git_repository_free(repo2);
 
@@ -79,10 +79,10 @@ void test_refs_create__deep_symbolic(void)
 	git_oid_fromstr(&id, current_master_tip);
 
 	cl_git_pass(git_buf_joinpath(&ref_path, g_repo->path_repository, new_head_tracker));
-	cl_git_pass(git_reference_create_symbolic(&new_reference, g_repo, new_head_tracker, current_head_target, 0));
+	cl_git_pass(git_reference_symbolic_create(&new_reference, g_repo, new_head_tracker, current_head_target, 0));
 	cl_git_pass(git_reference_lookup(&looked_up_ref, g_repo, new_head_tracker));
 	cl_git_pass(git_reference_resolve(&resolved_ref, looked_up_ref));
-	cl_assert(git_oid_cmp(&id, git_reference_oid(resolved_ref)) == 0);
+	cl_assert(git_oid_cmp(&id, git_reference_target(resolved_ref)) == 0);
 
 	git_reference_free(new_reference);
 	git_reference_free(looked_up_ref);
@@ -106,7 +106,7 @@ void test_refs_create__oid(void)
 	cl_git_pass(git_buf_joinpath(&ref_path, g_repo->path_repository, new_head));
 
 	/* Create and write the new object id reference */
-	cl_git_pass(git_reference_create_oid(&new_reference, g_repo, new_head, &id, 0));
+	cl_git_pass(git_reference_create(&new_reference, g_repo, new_head, &id, 0));
 
 	/* Ensure the reference can be looked-up... */
 	cl_git_pass(git_reference_lookup(&looked_up_ref, g_repo, new_head));
@@ -115,14 +115,14 @@ void test_refs_create__oid(void)
 	cl_assert_equal_s(looked_up_ref->name, new_head);
 
 	/* ...and that it points to the current master tip */
-	cl_assert(git_oid_cmp(&id, git_reference_oid(looked_up_ref)) == 0);
+	cl_assert(git_oid_cmp(&id, git_reference_target(looked_up_ref)) == 0);
 	git_reference_free(looked_up_ref);
 
 	/* Similar test with a fresh new repository */
 	cl_git_pass(git_repository_open(&repo2, "testrepo"));
 
 	cl_git_pass(git_reference_lookup(&looked_up_ref, repo2, new_head));
-	cl_assert(git_oid_cmp(&id, git_reference_oid(looked_up_ref)) == 0);
+	cl_assert(git_oid_cmp(&id, git_reference_target(looked_up_ref)) == 0);
 
 	git_repository_free(repo2);
 
@@ -142,7 +142,7 @@ void test_refs_create__oid_unknown(void)
 	git_oid_fromstr(&id, "deadbeef3f795b2b4353bcce3a527ad0a4f7f644");
 
 	/* Create and write the new object id reference */
-	cl_git_fail(git_reference_create_oid(&new_reference, g_repo, new_head, &id, 0));
+	cl_git_fail(git_reference_create(&new_reference, g_repo, new_head, &id, 0));
 
 	/* Ensure the reference can't be looked-up... */
 	cl_git_fail(git_reference_lookup(&looked_up_ref, g_repo, new_head));
@@ -156,9 +156,9 @@ void test_refs_create__propagate_eexists(void)
 
 	/* Make sure it works for oid and for symbolic both */
 	git_oid_fromstr(&oid, current_master_tip);
-	error = git_reference_create_oid(&ref, g_repo, current_head_target, &oid, false);
+	error = git_reference_create(&ref, g_repo, current_head_target, &oid, false);
 	cl_assert(error == GIT_EEXISTS);
 
-	error = git_reference_create_symbolic(&ref, g_repo, "HEAD", current_head_target, false);
+	error = git_reference_symbolic_create(&ref, g_repo, "HEAD", current_head_target, false);
 	cl_assert(error == GIT_EEXISTS);
 }
